// file: location_state.js
import * as locationService from "./services/location_service";
import * as ssService from "./services/storage/session_storage_service";

// An instance of this class is used by some views to store and restore their state,
// e.g. scroll position, texts of textboxes, etc. This is needed because it's SPA,
// i.e. pushState popState functionality is used.
export class LocationState {
    constructor() {
        // This property stores the previous location until a content is loaded by the view.
        // After the content is loaded, it must be updated (by the view) to the current location.
        this._locationHref = null;
    }

    updateLocation() {
        this._locationHref = locationService.getHref();
    }

    isLocationSet() {
        return this._locationHref !== null;
    }

    setState(state) {
        ssService.setItem(this._locationHref, state);
    }

    getState() {
        return ssService.getItem(this._locationHref);
    }
}