// file: main.js
import {} from "./utils";
import {enableToggles} from "./views";
import {BoardsSidebarV} from "./views/sidebars/boards_sidebar_v";
import {ThreadsSidebarV} from "./views/sidebars/threads_sidebar_v";
import {handlePopState} from "./services/history_navigation_service";
import {ChatContainerV} from "./views/chat_container_v";
import {enableKatex} from "./extra_services/katex_service";
import {initRemovePostPreviewsOnClick} from "./views/post/post_preview_v";
import * as locationService from "./services/location_service";
import * as repo from "./repository";
import {replaceByThreadPage} from "./services/location_service";
import {enableDrags} from "./views/drag/drag_v";
import {getSettings} from "./services/settings_service";
import {MenuV} from "./views/menu/menu_v";
import {enableHljs} from "./extra_services/hljs_service";
import {ContentFrameV} from "./views/content_frame_v";
import {getOrCreateClientId} from "./services/client_service";
import {config} from "./config";
import {initFavoritesService} from "./services/favorites_service";
import {initAliveTabsService} from "./services/alive_tabs_service";
import {initLocalStorageService} from "./services/storage/local_storage_service";
import {getThreadPageNumber} from "./services/page_number_service";
import {showNotification} from "./views/notifications/notification_v";
import {initThemesService} from "./services/themes_service";
import {initServerInfoService} from "./services/server_info_service";
import {newE} from "./utils";

document.addEventListener("DOMContentLoaded", () => {
    // todo: call just main when tor browser bug is fixed
    initLocalStorageService(main);
});

function main() {
    initAliveTabsService();
    initServerInfoService();
    repo.subscribe();
    new MenuV();
    //lsService.addStorageEventListener(); // todo: uncomment when tor browser bug is fixed
    getOrCreateClientId();

    enableToggles();
    enableDrags();

    const postId = locationService.getPostId();
    (postId ? repo.getPost(postId, true).then(post =>
                replaceByThreadPage(
                    post.thread.board.name,
                    post.thread.threadId,
                    getThreadPageNumber(post.index),
                    postId,
                )
            ).catch(e => {
                showNotification(config.notification.type.error, e.message);
                throw e;
            })
            : Promise.resolve()
    ).then(() => {
        new BoardsSidebarV();
        new ThreadsSidebarV();

        new ChatContainerV();

        Promise.all([
            enableKatex(getSettings().isKatexEnabled),
            enableHljs(getSettings().highlightCode),
        ]).then(() => {
            new ContentFrameV();
        });
    });

    const oldUiLinkE = document.createElement('a');
    oldUiLinkE.href = '/ui2/';
    oldUiLinkE.appendText(`The old ui is here: /ui2/`);
    showNotification(
        config.notification.type.info,
        newE(`div`, {style: `max-width: 300px`},
        `FYI: Ephemeral posting mode means that your posts don't get stored in the database,
        they are only retransmitted to other online users.
        It's the default posting mode for Anonymous.
        Create an account if you want your posts to be saved.
        More info can be found in the help.`),
        20000
    );

    addEventListener(`popstate`, (e) => {
        //console.log("location: " + document.location + ", state: ", e.state);

        handlePopState(e.state);
    }, false);

    initThemesService();

    initRemovePostPreviewsOnClick();
    if (!config.isClientWebUI) {
        initFavoritesService();
    }
}
