// file: models.js
import {config} from "./config";

export class Board {
    constructor(name, description, isFavorite) {
        this.name = name;
        this.description = description;
        this.isFavorite = isFavorite;
    }
}

export class Thread {
    constructor(threadId, subject, postCount, lastPostDate, board, isFavorite, lastSeenAt) {
        this.threadId = threadId;
        this.subject = subject;
        this.postCount = postCount;
        this.lastPostDate = lastPostDate;
        this.board = board;
        this.isFavorite = isFavorite;
        this.lastSeenAt = lastSeenAt;
    }
}

export class Post {
    constructor(postId, postDate, bg, fg, text, hiddenByOp, user, thread, index, ephemeral) {
        this.postId = postId;
        this.postDate = postDate;
        this.bg = bg;
        this.fg = fg;
        this.text = text;
        this.hiddenByOp = hiddenByOp;
        this.user = user;
        this.thread = thread;
        this.index = index; // the index of the post in the thread
        this.ephemeral = ephemeral;

        if (this.ephemeral && this.thread && this.thread.threadId === config.chat.threadId) {
            this.thread.board.name = config.chat.boardName;
        }
    }
}

export class ServerInfo {
    constructor(version, powBits) {
        this.version = version;
        this.powBits = powBits;
    }
}

export class ThreadSlice {
    constructor(thread, op, posts, prevPost) {
        this.thread = thread;
        this.op = op;
        this.posts = posts;
        this.prevPost = prevPost;
    }
}

export class WarpSlice {
    constructor(board, shellThreads) {
        this.board = board;
        this.shellThreads = shellThreads; // thread slices with less than 3 posts
    }
}

export class RecentPosts {
    constructor(chatThread, posts) {
        this.chatThread = chatThread;
        this.posts = posts;
    }
}

export class ShortUserInfo {
    constructor(nickname, banned) {
        this.nickname = nickname;
        this.banned = banned;
    }
}

export class NewPost {
    constructor(clientId, text, threadId, ephemeral, subject, boardName) {
        this.clientId = clientId;
        this.text = text;
        this.threadId = threadId;
        this.subject = subject;
        this.boardName = boardName;
        this.ephemeral = ephemeral;
    }
}

export class NewUser {
    constructor(nickname, password, pow) {
        this.nickname = nickname;
        this.password = password;
        this.pow = pow;
    }
}

export class Auth {
    constructor(nickname, password) {
        this.nickname = nickname;
        this.password = password;
    }
}

export class AuthInfo {
    constructor(accessToken) {
        this.accessToken = accessToken;
    }
}

export class Claims {
    constructor(name, iat) {
        this.name = name;
        this.iat = iat;
    }
}

export class UpdatedHiddenByOp {
    constructor(clientId, postId, hiddenByOp) {
        this.clientId = clientId;
        this.postId = postId;
        this.hiddenByOp = hiddenByOp;
    }
}

export class HiddenByOpEvent {
    constructor(postId, hiddenByOp) {
        this.postId = postId;
        this.hiddenByOp = hiddenByOp;
    }
}

export class SuccessResponse {
    constructor(data, skipped, taken, total) {
        this.data = data;
        this.skipped = skipped;
        this.taken = taken;
        this.total = total;
    }
}

export class RequestBody {
    constructor(data) {
        this.data = data;
    }
}

export class ErrorResponse {
    constructor(errorCode, message) {
        this.errorCode = errorCode;
        this.message = message;
    }
}

export class SearchArgs {
    constructor(query, range, orderDesc, boards, pageNumber) {
        this.query = query;
        this.range = range;
        this.orderDesc = orderDesc;
        this.boards = boards;
        this.pageNumber = pageNumber;
    }
}
