// file: parsers.js
import {
    AuthInfo,
    Board, Claims, ErrorResponse, HiddenByOpEvent,
    Post, RecentPosts, ServerInfo, ShortUserInfo, SuccessResponse, Thread, ThreadSlice, WarpSlice
} from "./models";

export function parseBoard(obj) {
    return new Board(
        obj.name,
        obj.description,
        obj.isFavorite,
    );
}

export function parseThread(obj) {
    return new Thread(
        obj.threadId,
        obj.subject,
        obj.postCount,
        obj.lastPostDate ? new Date(obj.lastPostDate) : null,
        obj.board ? parseBoard(obj.board) : null,
        obj.isFavorite,
        obj.lastSeenAt,
    );
}

export function parsePost(obj) {
    return new Post(
        obj.postId,
        new Date(obj.postDate),
        obj.bg,
        obj.fg,
        obj.text,
        obj.hiddenByOp,
        parseShortUserInfo(obj.user),
        obj.thread ? parseThread(obj.thread) : null,
        obj.index,
        obj.ephemeral,
    );
}

export function parseServerInfo(obj) {
    return new ServerInfo(
        obj.version,
        obj.powBits,
    );
}

export function parseThreadSlice(obj) {
    return new ThreadSlice(
        parseThread(obj.thread),
        parsePost(obj.op),
        obj.posts.map(parsePost),
        obj.prevPost ? parsePost(obj.prevPost) : null,
    );
}

export function parseWarpSlice(obj) {
    return new WarpSlice(
        obj.board ? parseBoard(obj.board) : null,
        obj.shellThreads.map(parseThreadSlice),
    );
}

export function parseShortUserInfo(obj) {
    return new ShortUserInfo(
        obj.nickname,
        obj.banned,
    );
}

export function parseAuthInfo(obj) {
    return new AuthInfo(
        obj.accessToken,
    );
}

export function parseClaims(accessToken) {
    const obj = JSON.parse(atob(accessToken.split('.')[1]));
    return new Claims(
        obj.name,
        obj.iat,
    );
}

export function parseSuccessResponse(obj) {
    return dataParser => {
        return new SuccessResponse(
            dataParser(obj.data),
            obj.skipped,
            obj.taken,
            obj.total,
        );
    }
}

export function parseErrorResponse(obj) {
    return new ErrorResponse(
        obj.errorCode,
        obj.message,
    );
}

export function parseRecentPosts(obj) {
    return new RecentPosts(
        parseThread(obj.chatThread),
        obj.posts.map(parsePost),
    )
}

export function parseHiddenByOpEvent(obj) {
    return new HiddenByOpEvent(
        obj.postId,
        obj.hiddenByOp,
    );
}