// file: client_service.js
import * as repo from "../repository";
import * as lsService from "./storage/local_storage_service";
import {config} from "../config";

export const clientIdLsKey = 'betaClientId';

export function getOrCreateClientId() {
    let clientId = lsService.getItem(clientIdLsKey, null);

    if (clientId) return Promise.resolve(clientId);

    if (config.isClientWebUI) {
        return repo.getClientId().then(id => lsService.setItem(clientIdLsKey, id));
    }

    clientId = newClientId();
    lsService.setItem(clientIdLsKey, clientId);

    return Promise.resolve(clientId);
}

export function updateClientId() {
    if (config.isClientWebUI) {
        return repo.updateClientId().then(id => lsService.setItem(clientIdLsKey, id))
    }

    const clientId = newClientId();
    lsService.setItem(clientIdLsKey, clientId);

    return Promise.resolve(clientId);
}

export function setClientId(clientId) {
    if (config.isClientWebUI) {
        throw new Error("Not implemented yet"); // todo
    }

    lsService.setItem(clientIdLsKey, clientId);

    return Promise.resolve(clientId);
}

function newClientId() {
    return Math.random().toString(36).substring(2);
}