// file: favorites_service.js
import * as lsService from "./storage/local_storage_service";
import {parseBoard, parseThread} from "../parsers";
import * as repo from "../repository";

export const favoriteBoardsLsKey = 'favorite_boards';

export function getFavoriteBoards() {
    return lsService.getItem(favoriteBoardsLsKey, []).map(parseBoard);
}

export function addOrRemoveFavoriteBoard(board) {
    let boards = getFavoriteBoards();

    if (board.isFavorite) {
        // add todo: there's probably a bug: you don't push in the correct order
        if (!boards.some(b => b.name === board.name)) boards.push(board);
    } else {
        // remove
        boards = boards.filter(b => b.name !== board.name)
    }

    lsService.setItem(favoriteBoardsLsKey, boards);
}


export const favoriteThreadsLsKey = 'favorite_threads';

export function getFavoriteThreads() {
    return lsService.getItem(favoriteThreadsLsKey, []).map(parseThread);
}

export function addOrRemoveFavoriteThread(thread) {
    let threads = getFavoriteThreads();

    if (thread.isFavorite) {
        // add todo: there's probably a bug: you don't push in the correct order
        if (!threads.some(t => t.threadId === thread.threadId)) threads.push(thread);
    } else {
        // remove
        threads = threads.filter(t => t.threadId !== thread.threadId);
    }

    lsService.setItem(favoriteThreadsLsKey, threads);
}

export function initFavoritesService() {
    repo.addBoardDescriptionListener((e, board) => {
        // change board description in LS
        lsService.setItem(favoriteBoardsLsKey, getFavoriteBoards().map(b => {
            if (b.name === board.name) b.description = board.description;
            return b;
        }));
    });
}
