// file: settings_service.js
import * as lsService from "./storage/local_storage_service";
import {config} from "../config";

let currentSettings = null;

export function getSettings() {
    if (!currentSettings) {
        currentSettings = lsService.getItem('settings', config.defaultSettings);
    }
    return currentSettings;
}

export function saveSettings(settings) {
    lsService.setItem('settings', settings);
    currentSettings = settings;
}

export function addSettingsChangeEventListener(onSettingsChange) {
    document.addEventListener('lsChange', e => {
        if (e.detail.key === 'settings') {
            const oldSettings = e.detail.oldValue || config.defaultSettings;
            const newSettings = e.detail.newValue;
            currentSettings = newSettings;

            onSettingsChange({oldSettings, newSettings});
        }
    }, false);
}

export function removeSettingsChangeEventListener(onSettingsChange) {
    document.removeEventListener('lsChange', onSettingsChange, false);
}