// file: title_service.js
import {config} from "../config";
import {toShortcutUuid} from "./uuid_service";
import {getPageNumber, getThreadPageNumber} from "./page_number_service";

let titlePrefix = ``;
let title = ``;

export function setTitlePrefix(prefix) {
    titlePrefix = prefix ? `${prefix} ` : ``;
    setTitle(title);
}

export function setTitle(txt) {
    title = txt;
    document.title = `${titlePrefix}${txt}`;
}

export function getBoardTitle(board) {
    return `/${board.name}/ – ${board.description}`;
}

export function getSearchTitle(searchResultPage) {
    const srp = searchResultPage;

    return `Search${getPageInfo(
        config.pageSize.general, srp.total, getPageNumber(srp.skipped)
    )}`;
}

export function getThreadPageTitle(threadPage) {
    const tp = threadPage;
    const t = threadPage.data.thread;

    return `/${t.board.name}/${toShortcutUuid(t.threadId)} – \`\`${t.subject}''${getPageInfo(
        config.pageSize.general, tp.total, getThreadPageNumber(tp.skipped)
    )}`;
}

export function getWarpTitle(warpPage) {
    const board = warpPage.data.board;
    const pageNumber = getPageNumber(warpPage.skipped, config.pageSize.warp);

    const pageInfo = pageNumber ? getPageInfo(config.pageSize.warp, warpPage.total, pageNumber) : ``;

    return (
        (board ? `/${board.name}/` : `pfpmd`) +
        (board && board.description ? ` – ${board.description}` : ``) +
        pageInfo
    );
}

function getPageInfo(pageSize, totalCount, pageNumber) {
    return pageSize >= totalCount ? `` : ` – page ${pageNumber}`;
}