import {decorateView, getComputedWidth} from "../../utils";

export class ChatContainerDragDependantD {
    constructor(dragDependantV) {
        decorateView(dragDependantV, this, 'dragDependantV');
        this.element = dragDependantV.element;

        this._dragDependantV = dragDependantV;
        this._containerE = $('#container');
    }

    setWidth(width) {
        if (this.element.view.isOnTop()) return;

        const currentWidth = this._getWidth();
        const availableWidth = this._getAvailableWidth();
        const delta = currentWidth - width;

        // if there will be no space left
        if (availableWidth + delta < 0) {
            width = currentWidth + availableWidth;
        }

        this._dragDependantV.setWidth(width);
    }

    setHeight(height) {
        if (this.element.view.isOnTop()) {
            this._dragDependantV.setHeight(height);
        }
    }

    toggleHide() {
        // in case of showing
        const availableWidth = this._getAvailableWidth();
        if (this.element.isHidden() && getComputedWidth(this.element) > availableWidth) {
            this.element.view.setWidth(availableWidth);
        }

        this._dragDependantV.toggleHide();
    }

    _getAvailableWidth() {
        return this._containerE.offsetWidth;
    }

    _getWidth() { return this.element.offsetWidth; }
    _getHeight() { return this.element.clientHeight; }
}