// file: chat_form_v.js
import {TextboxV} from "../textbox_v";
import {config} from "../../config";
import {ChatTextboxD} from "./chat_textbox_d";
import {getAccessToken} from "../../services/access_token_service";
import {addSettingsChangeEventListener, getSettings} from "../../services/settings_service";
import {isEphemeralPostingOn} from "../../services/ephemeral_posting_service";

export class ChatFormV {
    constructor(keepScrollBottomV) {
        const chatFormE = document.createElement('form');
        chatFormE.id = 'chat_form';

        this.element = chatFormE;
        chatFormE.chatFormV = this;

        this.tbxV = new TextboxV(() => keepScrollBottomV.keepScrollBottom());
        this.tbxV = new ChatTextboxD(this.tbxV);

        this.tbxV.autoResize();
        const tbxE = this.tbxV.element;
        tbxE.id = `chat_tbx`;
        tbxE.classList.add('chat_tbx');
        tbxE.maxLength = config.maxPostLength;
        tbxE.title = "Press Alt+Enter to start a new line";
        tbxE.autofocus = true;

        chatFormE.appendChild(tbxE);

        function onAuthorizeChange() {
            const isAuthorized = getAccessToken();
            // tbxE.disabled = !isAuthorized;
            tbxE.placeholder = isAuthorized ? "" : config.ephemeralModeWarning;
        }
        onAuthorizeChange();
        document.addEventListener('authorizeChange', e => onAuthorizeChange(), false);

        function onEphemeralPostingModeChange() {
            tbxE.placeholder = isEphemeralPostingOn() ? config.ephemeralModeWarning : "";
        }
        onEphemeralPostingModeChange();
        addSettingsChangeEventListener(e => {
            if (e.oldSettings.isEphemeralPostingOn !== e.newSettings.isEphemeralPostingOn) {
                onEphemeralPostingModeChange();
            }
        });
    }
}