// file: keep_scroll_bottom_v.js
export class KeepScrollBottomV {
    constructor(element, threshold) {
        this.element = element;
        this._threshold = threshold;

        this._clientWidth = element.clientWidth;
        this._clientHeight = element.clientHeight;

        this.element.scrollToBottom();
        this._wasScrolledToBottom = true;

        this._prevScrollTop = this.element.scrollTop;
        element.addEventListener("scroll", e => this._keepTrackOfScrollStatus(), false);

        this._running = false;
        window.addEventListener("resize", e => this._keepScrollBottomOnResize(), false);
        window.addEventListener("mouseup", e => this._onMouseUp(e), false);
        window.addEventListener("mousedown", e => this._onMouseDown(e), false);

        document.addEventListener('lsChange',
            this._onLsChangeHandler = e => this._onLsChange(e), false);
    }

    keepScrollBottom() {
        if (this._wasScrolledToBottom) this.element.scrollToBottom();
    }

    _onMouseUp(e) {
        this._isMouseDown = false;
        this._clientWidthOnMouseDown = null;
        this._clientHeightOnMouseDown = null;
        requestAnimationFrame(() => this.keepScrollBottom());
    }

    _onMouseDown(e) {
        this._isMouseDown = true;
        this._clientWidthOnMouseDown = this.element.clientWidth;
        this._clientHeightOnMouseDown = this.element.clientHeight;
    }

    _onLsChange(e) {
        // e.g. width might shrink in another tab
        if (e.detail.isFromForeignWindow) requestAnimationFrame(() => this.keepScrollBottom());
    }

    // _forEachAnimationFrame() {
    //     if (this._stopWatching) return;
    //
    //     this.keepScrollBottom();
    //     requestAnimationFrame(this._forEachAnimationFrameBound);
    // }

    _keepTrackOfScrollStatus() {
        if (this._isMouseDown && (
                this._clientWidthOnMouseDown !== this.element.clientWidth ||
                this._clientHeightOnMouseDown !== this.element.clientHeight)
        ) {
            // this is not the user scrolling, he's resizing
            return;
        }

        // workaround: for hidden elements, setting scrollTop doesn't do anything
        if (this._prevScrollTop === this.element.scrollTop) {
            // if we're here, scroll actually hasn't changed, just the element has become visible
            if (this._wasScrolledToBottom) {
                // while the element was hidden, we called scrollToBottom,
                // but it didn't do anything, but now we can actually change it
                this.element.scrollToBottom();
            }
            return;
        }
        this._prevScrollTop = this.element.scrollTop;

        // keep track whether the element is scrolled to the bottom or not
        this._wasScrolledToBottom = this._isScrolledToBottom();
    }

    _keepScrollBottomOnResize() {
        if (this._running) return;

        this._running = true;
        requestAnimationFrame(() => {
            this.keepScrollBottom();
            this._running = false;
        });
    }

    _isScrolledToBottom() {
        const el = this.element;
        return (el.scrollHeight - el.offsetHeight) - this._threshold <= el.scrollTop;
    }

    // _widthOrHeightShrunk() {
    //     const result = (this.element.clientWidth < this._clientWidth
    //         || this.element.clientHeight < this._clientHeight);
    //
    //     this._clientWidth = this.element.clientWidth;
    //     this._clientHeight = this.element.clientHeight;
    //
    //     return result;
    // }
}