// file: chat_container_v.js
import {DragDependantV} from "./drag/drag_depentant_v";
import {WidthStoredDragDependantD} from "./drag/width_stored_drag_dependent_d";
import {HeightStoredDragDependantD} from "./drag/height_stored_drag_dependent_d";
import {VisibilityStoredDragDependantD} from "./drag/visibility_stored_drag_dependent_d";
import {ChatContainerDragDependantD} from "./chat/chat_container_drag_dependant_d";
import {DraggableWindowV} from "./draggable_window_v";
import * as lsService from "../services/storage/local_storage_service";
import {parseCssDimension} from "../utils";
import {chatVisibilityChange} from "../custom_events";
import {ChatContainerTrackVisibilityD} from "./chat/chat_container_track_visibility_d";

export class ChatContainerV {
    constructor() {
        const chatContainerE = $('#chat_container');
        chatContainerE.view = this;
        this.element = chatContainerE;

        this.dragE = $('#chat_container_drag');

        this.CHAT_CONTAINER_DETACHED = 'chat_container_detached';
        this.DETACHED_CHAT_CONTAINER_WIDTH = 'detached_chat_container_width';
        this.DETACHED_CHAT_CONTAINER_HEIGHT = 'detached_chat_container_height';
        this.CHAT_CONTAINER_LEFT = 'chat_container_left';
        this.CHAT_CONTAINER_TOP = 'chat_container_top';
        this.ATTACHED_CHAT_CONTAINER_WIDTH = 'attached_chat_container_width';
        this.ATTACHED_CHAT_CONTAINER_HEIGHT = 'attached_chat_container_height';
        this.CHAT_CONTAINER_HIDDEN = 'chat_container_hidden';
        this.CHAT_CONTAINER_ON_TOP = 'chat_container_on_top';
        this.CHAT_CONTAINER_COLLAPSED = 'chat_container_collapsed';

        this.dragDependantV = new DragDependantV(chatContainerE);
        this.dragDependantV = new ChatContainerDragDependantD(this.dragDependantV);
        this.dragDependantV = new VisibilityStoredDragDependantD(
            this.dragDependantV, this.CHAT_CONTAINER_HIDDEN, false
        );
        this.dragDependantV = new WidthStoredDragDependantD(
            this.dragDependantV, this.ATTACHED_CHAT_CONTAINER_WIDTH, window.innerWidth / 2
        );
        this.dragDependantV = new HeightStoredDragDependantD(
            this.dragDependantV, this.ATTACHED_CHAT_CONTAINER_HEIGHT, window.innerHeight / 2
        );
        this.dragDependantV = new ChatContainerTrackVisibilityD(this.dragDependantV);

        this.draggableWindowV = new DraggableWindowV(chatContainerE);

        this.titleBarE = $('#chat_title_bar');
        this.detachBtnE = this.titleBarE.$('.detach_btn');
        this.detachBtnE.addEventListener('click', e => this.toggleDetach(e), false);

        this.moveUpBtnE = this.titleBarE.$('.move_up_btn');
        this.moveRightBtnE = this.titleBarE.$('.move_right_btn');
        this.contentFrameE = $('#content_frame');
        this.moveUpBtnE.addEventListener('click', e => this.moveUp(), false);
        this.moveRightBtnE.addEventListener('click', e => this.moveRight(), false);

        this.collapseBtnE = this.titleBarE.$('.collapse_btn');
        this.expandBtnE = this.titleBarE.$('.expand_btn');
        this.collapseBtnE.addEventListener('click', e => this.collapse(), false);
        this.expandBtnE.addEventListener('click', e => this.expand(), false);

        if (this.isDetached()) this.detach(); else this.attach();

        this.saveDetachedStateOnMouseUp();

        document.addEventListener('lsChange',
            this._onLsChangeHandler = e => this._onLsChange(e.detail), false);
    }

    toggleDetach(e) {
        if (lsService.getItem(this.CHAT_CONTAINER_DETACHED)) this.attach(); else this.detach();
    }

    detach() {
        lsService.setItem(this.CHAT_CONTAINER_DETACHED, true);

        this.element.style.minWidth = "";
        this.element.style.minHeight = "";
        this.element.style.width = `${this._getWidth()}px`;
        this.element.style.height = `${this._getHeight()}px`;

        this.element.style.left = `${this._getLeft()}px`;
        this.element.style.top = `${this._getTop()}px`;

        this.element.classList.add('detached');
        this.dragE.hide();

        this.draggableWindowV.isDisabled = false;
        this.detachBtnE.title = "attach";

        // change the icon
        const icon = this.detachBtnE.$(".icon");
        icon.classList.remove('icon-popout');
        icon.classList.add('icon-popin');

        this.moveUpBtnE.hide();
        this.moveRightBtnE.hide();

        if (this.isCollapsed()) this.collapse(); else this.expand();
    }

    attach() {
        lsService.setItem(this.CHAT_CONTAINER_DETACHED, false);

        if (this.isOnTop()) this.moveUp(); else this.moveRight();

        this.element.classList.remove('detached');
        this.dragE.show();

        this.draggableWindowV.isDisabled = true;
        this.detachBtnE.title = "detach";

        // change the icon
        const icon = this.detachBtnE.$(".icon");
        icon.classList.remove('icon-popin');
        icon.classList.add('icon-popout');

        const isOnTop = lsService.getItem(this.CHAT_CONTAINER_ON_TOP, false);
        this.moveUpBtnE.showOrHide(!isOnTop);
        this.moveRightBtnE.showOrHide(isOnTop);
    }

    saveDetachedStateOnMouseUp() {
        document.addEventListener("mouseup", () => {
            if (!this.isDetached()) return;

            const style = getComputedStyle(this.element);

            lsService.setItem(this.DETACHED_CHAT_CONTAINER_WIDTH, parseCssDimension(style.width));
            lsService.setItem(this.DETACHED_CHAT_CONTAINER_HEIGHT, parseCssDimension(style.height));
            lsService.setItem(this.CHAT_CONTAINER_LEFT, parseCssDimension(style.left));

            if (!this.isCollapsed()) {
                lsService.setItem(this.CHAT_CONTAINER_TOP, parseCssDimension(style.top));
            }
        }, false);
    }

    moveUp() {
        lsService.setItem(this.CHAT_CONTAINER_ON_TOP, true);

        // this.contentFrameE.style.flexDirection = "column-reverse";
        this.contentFrameE.classList.add('chat_on_top');

        const height = lsService.getItem(this.ATTACHED_CHAT_CONTAINER_HEIGHT, null);
        this.element.style.height = height ? `${height}px` : `40vh`;
        this.element.style.minHeight = this.element.style.height.repeat(1);
        this.element.style.minWidth = "";
        this.element.style.width = "";

        this.moveUpBtnE.hide();
        this.moveRightBtnE.show();

        const icon = this.dragE.$(".icon");
        icon.classList.remove("icon-vdots");
        icon.classList.add("icon-hdots");

        this.dragE.dragV.isVertical = false;
        this.dragE.dragV.direction = 1; // because flexDirection = "column-reverse"
    }

    moveRight() {
        lsService.setItem(this.CHAT_CONTAINER_ON_TOP, false);

        // this.contentFrameE.style.flexDirection = "row";
        this.contentFrameE.classList.remove('chat_on_top');

        const width = lsService.getItem(this.ATTACHED_CHAT_CONTAINER_WIDTH, window.innerWidth / 2);
        this.dragDependantV.setWidth(width);
        this.element.style.height = "";
        this.element.style.minHeight = "";

        this.moveUpBtnE.show();
        this.moveRightBtnE.hide();

        const icon = this.dragE.$(".icon");
        icon.classList.remove("icon-hdots");
        icon.classList.add("icon-vdots");

        this.dragE.dragV.isVertical = true;
        this.dragE.dragV.direction = -1;
    }

    collapse() {
        lsService.setItem(this.CHAT_CONTAINER_COLLAPSED, true);

        this.element.classList.add('chat_collapsed');
        this.element.style.top = `calc(100% - var(--menu-height))`;

        this.draggableWindowV.setTopFixed(true);
        chatVisibilityChange(true);
    }

    expand() {
        lsService.setItem(this.CHAT_CONTAINER_COLLAPSED, false);

        this.element.classList.remove('chat_collapsed');
        this.element.style.top = `${this._getTop()}px`;

        this.draggableWindowV.setTopFixed(false);
        chatVisibilityChange(false);
    }

    isDetached() {
        return lsService.getItem(this.CHAT_CONTAINER_DETACHED, false);
    }

    isOnTop() {
        return lsService.getItem(this.CHAT_CONTAINER_ON_TOP, false);
    }

    isCollapsed() {
        return this.isDetached() && lsService.getItem(this.CHAT_CONTAINER_COLLAPSED, false);
    }

    _getWidth() {
        return lsService.getItem(this.DETACHED_CHAT_CONTAINER_WIDTH, window.innerWidth / 2);
    }

    _getHeight() {
        return lsService.getItem(this.DETACHED_CHAT_CONTAINER_HEIGHT, window.innerHeight / 2);
    }

    _getLeft() {
        return lsService.getItem(
            this.CHAT_CONTAINER_LEFT,
            Math.max(0, window.innerWidth - this._getWidth()) / 2
        );
    }

    _getTop() {
        return lsService.getItem(
            this.CHAT_CONTAINER_TOP,
            Math.max(0, window.innerHeight - this._getHeight()) / 2
        );
    }

    _onLsChange(e) {
        if (!e.isFromForeignWindow) return;

        if (e.key === this.CHAT_CONTAINER_DETACHED) {
            e.newValue ? this.detach() : this.attach();
        }

        if (e.key === this.CHAT_CONTAINER_ON_TOP) {
            e.newValue ? this.moveUp() : this.moveRight();
        }

        if (e.key === this.CHAT_CONTAINER_COLLAPSED) {
            e.newValue ? this.collapse() : this.expand();
        }

        if (e.key === this.DETACHED_CHAT_CONTAINER_WIDTH ||
            e.key === this.DETACHED_CHAT_CONTAINER_HEIGHT ||
            e.key === this.CHAT_CONTAINER_LEFT ||
            e.key === this.CHAT_CONTAINER_TOP
        ) {
            if (this.element.classList.contains('detached')) {
                this.detach();
            }
        }
    }
}

export function isChatOrDocumentHidden() {
    return document.hidden || isChatHidden();
}

export function isChatHidden() {
    const chatContainerE = $('#chat_container');
    return chatContainerE.isHidden() || chatContainerE.view.isCollapsed();
}