// file: content_frame_v.js
import {ContainerV} from "./container_v";
import {ChatBodyV} from "./chat/chat_body_v";
import * as repo from "../repository";
import {addSettingsChangeEventListener} from "../services/settings_service";

export class ContentFrameV {
    constructor() {
        const contentFrameE = $('#content_frame');

        this.element = contentFrameE;
        contentFrameE.contentFrameV = this;

        const containerV = new ContainerV();
        const chatBodyV = new ChatBodyV();

        addSettingsChangeEventListener(this._onSettingsChangeHandler = e => {
            this._onSettingsChange(e.oldSettings, e.newSettings);
        });
    }

    _onSettingsChange(oldSettings, settings) {
        if (oldSettings.trackNewPosts !== settings.trackNewPosts) {
            // we place it here and not in the chat and container to avoid calling it twice
            if (settings.trackNewPosts) {
                // for all threads sets lastSeenAt to the latest post date of the thread
                repo.resetAllLastSeenAt();
            } else {
                // for all threads sets lastSeenAt to null
                repo.deleteAllLastSeenAt();
            }
        }
    }
}