import {getComputedHeight, getComputedWidth} from "../../utils";

export class DragV {
    constructor(dragE, isVertical) {
        this.element = dragE;
        dragE.dragV = this;

        // for vertical drags we resize width
        // for horizontal drags we resize height
        this.isVertical = isVertical; // false means horizontal

        const previousOrNext = dragE.getAttribute('dependent_sibling');
        this.dependentE = dragE[`${previousOrNext}ElementSibling`];
        this.direction = previousOrNext === 'next' ? -1 : 1;

        this._initialClientX = null;
        this._initialClientY = null;
        this._initialWidth = null;
        this._initialHeight = null;
        this._isResized = false;

        dragE.addEventListener("mousedown",
            this._onMouseDownHandler = e => this._onMouseDown(e), false);

        window.addEventListener("mouseup",
            this._onMouseUpHandler = e => this._onMouseUp(e), false);

        document.body.addEventListener("mousemove",
            this._onMouseMoveHandler = e => this._onMouseMove(e), false);
    }

    _onMouseDown(e) {
        e.preventDefault();

        // start resizing
        this._initialClientX = e.clientX;
        this._initialClientY = e.clientY;
        this._initialWidth = getComputedWidth(this.dependentE);
        this._initialHeight = getComputedHeight(this.dependentE);
    }

    _onMouseUp(e) {
        // if "mousedown" happened for this drag and it hasn't been resized yet.
        if (this._isMouseDown() && !this._isResized) {
            this.dependentE.dragDependantV.toggleHide();
        }
        this._isResized = false;

        // stop resizing
        this._stopResizing();
    }

    _onMouseMove(e) {
        if (!this._isMouseDown()) return;
        e.preventDefault();

        if (this.dependentE.isHidden()) {
            this.dependentE.dragDependantV.toggleHide();
            this._initialWidth = 0;
            this._initialHeight = 0;
        }

        const widthDelta = e.clientX - this._initialClientX;
        const heightDelta = e.clientY - this._initialClientY;
        const width = Math.max(0, this._initialWidth + this.direction * widthDelta);
        const height = Math.max(0, this._initialHeight + this.direction * heightDelta);

        if (this.isVertical) {
            this.dependentE.dragDependantV.setWidth(width);
        } else {
            this.dependentE.dragDependantV.setHeight(height);
        }

        this._isResized = true;
    }

    _isMouseDown() {
        return !!this._initialClientX && !!this._initialClientY;
    }
    _stopResizing() {
        this._initialClientX = null;
        this._initialClientY = null;
    }
}

export function enableDrags() {
    $a(".drag").each(dragE => new DragV(dragE, true));
}