export class DraggableWindowV {
    constructor(element) {
        this.element = element;
        this.isDisabled = false;

        this.titleBarE = element.$('.title_bar');
        if (!this.titleBarE) throw new Error("There must be a title bar!");

        this.titleBarE.addEventListener("mousedown",
            this._onMouseDownHandler = e => this.onMouseDown(e), false);
    }

    onMouseDown(e) {
        if (this.isDisabled) return;
        e.preventDefault();

        this.xBefore = e.clientX;
        this.yBefore = e.clientY;

        document.addEventListener("mouseup",
            this._onMouseUpHandler = e => this.onMouseUp(e), false);

        document.addEventListener("mousemove",
            this._onMouseMoveHandler = e => this.onMouseMove(e), false);
    }

    onMouseUp(e) {
        document.removeEventListener("mouseup", this._onMouseUpHandler, false);
        document.removeEventListener("mousemove", this._onMouseMoveHandler, false);
    }

    onMouseMove(e) {
        e.preventDefault();

        const xDelta = e.clientX - this.xBefore;
        const yDelta = e.clientY - this.yBefore;

        const left = Math.min(
            Math.max(0, this.element.offsetLeft + xDelta),
            window.innerWidth - this.element.offsetWidth
        );
        const top = Math.min(
            Math.max(0, this.element.offsetTop + yDelta),
            window.innerHeight - this.titleBarE.offsetHeight
        );

        this.element.style.left = `${left}px`;
        if (!this._isTopFixed) this.element.style.top = `${top}px`;

        this.xBefore = e.clientX;
        if (!this._isTopFixed) this.yBefore = e.clientY;
    }

    setTopFixed(isFixed) {
        this._isTopFixed = isFixed;
    }
}