// file: filter_service.js
export function containCyrillic(str) {
    for (let i = 0; i < str.length; i++) {
        const charCode = str.charCodeAt(i);
        if (charCode > 1023 && charCode < 1280) return true;
    }
    return false;
}

export function getShannonEntropy(text) {
    const set = {};

    text.split('').forEach(c => set[c] ? set[c]++ : (set[c] = 1));

    return Object.keys(set).reduce((acc, c) => {
        let p = set[c] / text.length;
        return acc - (p * (Math.log(p) / Math.log(2)));
    }, 0);
}


let regexpFiltersStr = '';
let filtersRegexps = []; // todo: use Set
export function parseRegexpFilters(str) {
    // we memorize the last result because we don't want to parse the same regexps for each post
    if (str === regexpFiltersStr) return filtersRegexps;

    filtersRegexps = [];
    regexpFiltersStr = (str || '');

    regexpFiltersStr.split('\n').filter(x => x).forEach(x => {
        try {
            filtersRegexps.push(new RegExp(x, 'i'));
        } catch (e) {
            console.warn("Invalid regexp: ", x, e)
        }
    });

    return filtersRegexps;
}

let colorsFiltersStr = '';
let colorsFilters = [];
export function parseColorsFilters(str) {
    if (str === colorsFiltersStr) return colorsFilters;

    colorsFilters = [];
    colorsFiltersStr = (str || '');

    const colorsFiltersSet = new Set();

    colorsFiltersStr.split('\n').filter(x => x).forEach(x => colorsFiltersSet.add(x));

    colorsFilters = Array.from(colorsFiltersSet).map(x => {
        const match = /^([0-9]+):([0-9]+)$/.exec(x);
        if (!match) return;

        return {
            bg: Number.parseInt(match[1]),
            fg: Number.parseInt(match[2]),
        };
    }).filter(x => x);

    return colorsFilters;
}