// file: import_btn_v.js
import {showNotification} from "../notifications/notification_v";
import {config} from "../../config";
import * as lsService from "../../services/storage/local_storage_service";

export class ImportBtnV {
    constructor() {
        const btnE = $("#import_btn");
        this.element = btnE;
        this.importBtnV = this;

        btnE.addEventListener('click', e => uploadUiState(), false);
    }
}

function importUiState(uiState) {
    // validate uiState
    if (!uiState.ui) {
        showError(`Missing key: "ui".`);
        return;
    }
    if (uiState.ui !== "ui") {
        showError(`Wrong ui value: expected "ui"; got "${uiState.ui}".`);
        return;
    }
    if (!uiState.version) {
        showError(`Missing key: "version".`);
        return;
    }
    if (uiState.version.constructor !== "".constructor) {
        showError(`"version" must be a string`);
        return;
    }
    if (!/^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)$/.test(uiState.version)) {
        showError(
            `Wrong version value: "${uiState.version}" doesn't match 
            "^(0|[1-9]\\d*)\\.(0|[1-9]\\d*)\\.(0|[1-9]\\d*)$" pattern."`
        );
        return;
    }
    if (!uiState.localStorage) {
        showError(`Missing key: "localStorage".`);
        return;
    }
    if (uiState.localStorage.constructor !== {}.constructor) {
        showError(`"localStorage" must be an object.`);
        return;
    }

    Object.keys(uiState.localStorage).map(
        key => lsService.setItem(key, uiState.localStorage[key], undefined, true)
    );
}

function uploadUiState() {
    const inputE = document.createElement("input");
    inputE.hide();
    inputE.type = "file";
    inputE.accept = "application/json";
    document.body.appendChild(inputE);

    inputE.addEventListener('change', e => {
        const reader = new FileReader();

        reader.onload = event => {
            let uiState;
            try {
                uiState = JSON.parse(event.target.result);
            } catch (err) {
                showError(`The file must be a json file.`);
                return;
            }
            importUiState(uiState);
        };

        reader.readAsText(e.target.files[0]);
        inputE.remove();
    }, false);

    inputE.click();
}

function showError(message) {
    return showNotification(config.notification.type.error, message, 5000);
}