// file: pagination_v.js
import {range} from "../../utils";
import {PaginationGapV} from "./pagination_gap_v";

export class PaginationV {
    constructor(totalCount, currentPage, pageSize, siblingCount, pageLinkF) {
        this._pageSize = pageSize;
        this._currentPage = currentPage;
        this._siblingCount = siblingCount;
        this._pageLinkF = pageLinkF;

        // Note: pages start from 0.
        const paginationE = document.createElement('div');
        paginationE.classList.add('pagination');

        paginationE.paginationV = this;
        this.element = paginationE;

        this._addPaginationElements(totalCount, currentPage, siblingCount, pageLinkF);
    }

    onTotalCountChange(newTotalCount) {
        const newPageCount = Math.ceil(newTotalCount / this._pageSize);

        // if page count didn't change, then do nothing
        if (newPageCount === this._pageCount) return;

        // recreate elements
        this.element.empty();
        this._addPaginationElements(
            newTotalCount, this._currentPage, this._siblingCount, this._pageLinkF
        );
    }

    _addPaginationElements(totalCount, currentPage, siblingCount, pageLinkF) {
        this._pageCount = Math.ceil(totalCount / this._pageSize);
        if (this._pageCount <= 1) return;

        if (currentPage !== 0) pageLinkF.create(0, currentPage, this);

        const lastPageNumber = this._pageCount - 1;

        // 0 . 2 3 4 [5]
        const isLeftGapNeeded = currentPage - siblingCount - 1 > 0;
        if (isLeftGapNeeded) new PaginationGapV(this);

        range(siblingCount)
            .map((_, i) => currentPage - i - 1)
            .reverse()
            .filter(pageNumber => pageNumber > 0)
            .forEach(pageNumber => pageLinkF.create(pageNumber, currentPage, this));

        pageLinkF.create(currentPage, currentPage, this);

        range(siblingCount)
            .map((_, i) => currentPage + i + 1)
            .filter(pageNumber => pageNumber < lastPageNumber)
            .forEach(pageNumber => pageLinkF.create(pageNumber, currentPage, this));

        // [5] 6 7 8 . 10
        const isRightGapNeeded = currentPage + siblingCount + 1 < lastPageNumber;
        if (isRightGapNeeded) new PaginationGapV(this);

        if (currentPage === lastPageNumber) return;
        const lastPageLinkV = pageLinkF.create(lastPageNumber, currentPage, this);
    }
}