// file: popup_saving_state_d.js
import {decorateView, parseCssDimension} from "../../utils";
import * as lsService from "../../services/storage/local_storage_service";

export class PopupSavingStateD {
    constructor(popupV, defaultWidth = 600, defaultHeight = 500) {
        const popupId = popupV.element.id;

        if (!popupId) throw new Error('The popup must have an id!');

        decorateView(popupV, this, 'popupV');
        this.element = popupV.element;

        this.lsKeyWidth = `popup_${popupId}_width`;
        this.lsKeyHeight = `popup_${popupId}_height`;
        this.lsKeyLeft = `popup_${popupId}_left`;
        this.lsKeyTop = `popup_${popupId}_top`;

        const width = lsService.getItem(
            this.lsKeyWidth,
            Math.min(defaultWidth, window.innerWidth)
        );
        const height = lsService.getItem(
            this.lsKeyHeight,
            Math.min(defaultHeight, window.innerHeight)
        );

        this.element.style.width = `${width}px`;
        this.element.style.height = `${height}px`;

        const left = lsService.getItem(
            this.lsKeyLeft,
            Math.max(0, window.innerWidth - width) / 2
        );
        const top = lsService.getItem(
            this.lsKeyTop,
            Math.max(0, window.innerHeight - height) / 2
        );

        this.element.style.left = `${left}px`;
        this.element.style.top = `${top}px`;

        document.addEventListener('mouseup', this._onMouseUpHandler = () => {
            this.saveStateOnMouseUp();
        }, false);

        document.addEventListener('lsChange',
            this._onLsChangeHandler = e => this._onLsChange(e.detail), false);

        this.close = () => {
            document.removeEventListener('mouseup', this._onMouseUpHandler, false);
            document.removeEventListener('lsChange', this._onLsChangeHandler, false);
            popupV.close();
        };
    }

    saveStateOnMouseUp() {
        const style = getComputedStyle(this.element);

        lsService.setItem(this.lsKeyWidth, parseCssDimension(style.width));
        lsService.setItem(this.lsKeyHeight, parseCssDimension(style.height));
        lsService.setItem(this.lsKeyLeft, parseCssDimension(style.left));
        lsService.setItem(this.lsKeyTop, parseCssDimension(style.top));
    }

    _onLsChange(e) {
        if (!e.isFromForeignWindow) return;

        if (e.key === this.lsKeyWidth) this.element.style.width = `${e.newValue}px`;
        if (e.key === this.lsKeyHeight) this.element.style.height = `${e.newValue}px`;
        if (e.key === this.lsKeyLeft) this.element.style.left = `${e.newValue}px`;
        if (e.key === this.lsKeyTop) this.element.style.top = `${e.newValue}px`;
    }
}