// file: link_v.js
import {tryDecodeURI} from "../../utils";
import {
    getImgurThumbnailV, getLibreVideoThumbnailV, getPloudfrThumbnailV, getTwitterThumbnailV,
    getYoutubeThumbnailV, getPeertubeSocialThumbnailV,
} from "./thumbnail_v";
import {getSettings} from "../../services/settings_service";
import {getBitchuteEmbedV} from "./embed_v";

export class LinkV {
    constructor(url, name) {
        const aE = document.createElement("a");
        aE.target = '_blank';
        aE.href = url;
        aE.referrerPolicy = "no-referrer";
        aE.rel = "noreferrer";
        aE.textContent = name ? name : tryDecodeURI(url);

        this.element = aE;
        aE.linkV = this;

        formatLink(aE, url);
    }
}

function formatLink(aE, url) {
    let settings = getSettings();

    if (settings.embedBitchute) {
        let thumbV = getBitchuteEmbedV(url);
        if (thumbV) {
            addThumb(thumbV, "bitchute_link");
            aE.appendChild(document.createElement("br"));
            aE.appendText(tryDecodeURI(url));
            return;
        }
    }

    if (!settings.showImages) {
        return;
    }

    let thumbV = getYoutubeThumbnailV(url);
    if (thumbV) {
        addThumb(thumbV, "youtube_link");
        return;
    }
    thumbV = getImgurThumbnailV(url);
    if (thumbV) {
        // todo: refactor this block
        addThumb(thumbV, "imgur_link");
        // aE.id = Date.now();

        // thumbV.onload = function () {
        //     aE = document.getElementById(aE.id);
        //     var msgDiv = getClosestByClassName(aE, "msg");
        //     var msgHeightBefore = parseInt(getComputedStyle(msgDiv).height);

        //     addThumb(thumbV, "imgur-link");

        //     var msgHeightAfter = parseInt(getComputedStyle(msgDiv).height);
        //     var addedHeight = msgHeightAfter - msgHeightBefore;

        //     var chatFrameDiv = document.getElementById("chatframe");
        //     chatFrameDiv.scrollTop += addedHeight;
        // };
        return;
    }
    thumbV = getTwitterThumbnailV(url);
    if (thumbV) {
        addThumb(thumbV, "twitter_link");
        return;
    }
    thumbV = getPloudfrThumbnailV(url);
    if (thumbV) {
        addThumb(thumbV, "ploudfr_link");
        return;
    }
    thumbV = getLibreVideoThumbnailV(url);
    if (thumbV) {
        addThumb(thumbV, "librevideo_link");
        return;
    }
    thumbV = getPeertubeSocialThumbnailV(url);
    if (thumbV) {
        addThumb(thumbV, "peertubesocial_link");
        return;
    }

    function addThumb(thumbV, className) {
        aE.textContent = "";
        aE.classList.add(className);
        aE.appendChild(thumbV.element);
    }
}