// post_link_v.js
import * as repo from "../../repository";
import * as locationService from "../../services/location_service";
import {getPostLinkUrl} from "../../services/url_service";
import {getThreadPageNumber} from "../../services/page_number_service";
import {showNotification} from "../notifications/notification_v";
import {config} from "../../config";

export class PostLinkV {
    constructor(postId, linkText, title) {
        this.postId = postId;

        const linkE = document.createElement("a");
        linkE.href = getPostLinkUrl(postId);
        if (title) linkE.title = title;
        linkE.appendText(linkText);

        this.element = linkE;
        linkE.postLinkV = this;

        linkE.addEventListener('click', e => this._onClick(e), false);
    }

    _onClick(e) {
        e.preventDefault();

        const hash = `#p${this.postId}`;
        const postE = $(hash);
        if (postE && locationService.getThreadId()) {
            locationService.setPostIdHash(hash);
            return;
        }

        repo.getPost(this.postId, true).then(post =>
            locationService.setThreadPage(
                post.thread.board.name,
                post.thread.threadId,
                getThreadPageNumber(post.index),
                this.postId,
            )
        ).catch(e => showNotification(config.notification.type.error, e.message));
    }
}