// file: post_preview_v.js
import {PostV} from "./post_v";

export class PostPreviewV {
    constructor(post, rect) {
        const postPreviewE = document.createElement('div');
        postPreviewE.classList.add('post_preview');

        this.element = postPreviewE;
        postPreviewE.postPreviewV = this;

        const postV = new PostV(post);
        postPreviewE.appendChild(postV.element);
        document.body.appendChild(postPreviewE);

        const totalWidth = document.documentElement.clientWidth;
        const totalHeight = document.documentElement.clientHeight;

        const onLeft = rect.left + rect.width/2 <= totalWidth/2;
        const onTop = rect.top + rect.height/2 <= totalHeight/2;


        if (onLeft) {
            postPreviewE.style.maxWidth = Math.floor(totalWidth - rect.left) + "px";
            postPreviewE.style.left = rect.left + "px";
        } else {
            postPreviewE.style.maxWidth = Math.floor(rect.right) + "px";
            postPreviewE.style.right = (totalWidth - rect.right) + "px";
        }

        if (onTop) {
            postPreviewE.style.maxHeight = Math.floor(totalHeight - rect.bottom) + "px";
            postPreviewE.style.top = rect.bottom + "px";
        } else {
            postPreviewE.style.maxHeight = Math.floor(rect.top) + "px";
            postPreviewE.style.top = (rect.top - postPreviewE.offsetHeight) + "px";
        }

        postPreviewE.addEventListener('mouseleave', e => this._onMouseLeave(e), false);
    }

    remove() {
        this.element.remove();
        this.element = null;
    }

    _onMouseLeave(e) {
        if (!e.relatedTarget) return;
        if (e.relatedTarget.closest('.post_ref')) return;

        const relatedPostPreviewE = e.relatedTarget.closest('.post_preview');
        const postPreviewsEs = $a('.post_preview');

        const indexOfThisPostPreview = postPreviewsEs.indexOf(this.element);
        const indexOfRelatedPostPreview = postPreviewsEs.indexOf(relatedPostPreviewE);

        if (indexOfRelatedPostPreview > indexOfThisPostPreview) return;

        for (let i = postPreviewsEs.length - 1; i >= indexOfThisPostPreview; i--) {
            postPreviewsEs[i].postPreviewV.remove();
        }
    }
}

export function initRemovePostPreviewsOnClick() {
    document.body.addEventListener('click', e => {
        if (e.target.closest('.post_preview')) return;

        $a('.post_preview').each(postPreviewE => postPreviewE.postPreviewV.remove());
    });
}