// file: post_reference_v.js
import * as repo from "../../repository";
import {PostPreviewV} from "./post_preview_v";
import {PostLinkV} from "./post_link_v";
import {toShortcutUuid} from "../../services/uuid_service";
import {showNotification} from "../notifications/notification_v";
import {config} from "../../config";

export class PostReferenceV {
    constructor(postId) {
        this.postId = postId;
        this._loadPostPreviewTimeout = 100;
        this._requestTimetoutId = null;

        const refE = document.createElement("span");
        refE.classList.add("post_ref");

        this.element = refE;
        refE.postReferenceV = this;

        const postLinkV = new PostLinkV(postId, `@${toShortcutUuid(postId)}`);
        this._linkE = postLinkV.element;
        refE.appendChild(this._linkE);

        refE.addEventListener('mouseenter', e => this._onMouseEnter(e), false);
        refE.addEventListener('mouseleave', e => this._onMouseLeave(e), false);
    }

    _onMouseEnter(e) {
        if (this._postPreviewExists() || this._requestTimetoutId) return;

        this._linkE.style.cursor = 'wait';

        this._requestTimetoutId = setTimeout(() => {
            repo.getPost(this.postId).then(p => {
                if (!this._requestTimetoutId) return;

                this._showPostPreview(p);
                this._requestTimetoutId = null;
                this._linkE.style.cursor = ``;
            }).catch(e => {
                this._linkE.style.cursor = ``;
                showNotification(config.notification.type.error, e.message);
            });
        }, this._loadPostPreviewTimeout);
    }

    _onMouseLeave(e) {
        clearTimeout(this._requestTimetoutId);
        this._requestTimetoutId = null;

        if (!this._postPreviewExists() || !e.relatedTarget) return;

        const relatedPostPreviewE = e.relatedTarget.closest('.post_preview');
        if (relatedPostPreviewE === this._postPreviewV.element) return;

        this._removePostPreview();
    }

    _showPostPreview(post) {
        const rect = this.element.getBoundingClientRect();

        this._postPreviewV = new PostPreviewV(post, rect);
    }

    _removePostPreview() {
        if (!this._postPreviewExists()) return;

        this._postPreviewV.remove();
        this._postPreviewV = null;
    }

    _postPreviewExists() {
        return this._postPreviewV && this._postPreviewV.element;
    }
}