// file: thumbnail_v.js
export class ThumbnailV {
    constructor(url) {
        const pictureE = document.createElement("picture");

        this.element = pictureE;
        pictureE.thumbnailV = this;

        this.imgE = document.createElement("img");
        this.imgE.referrerPolicy = "no-referrer";
        this.imgE.title = url;
        this.imgE.alt = url;

        pictureE.appendChild(this.imgE);
    }
}

export function getYoutubeThumbnailV(url) {
    const videoId = getYoutubeVideoId(url);
    if (!videoId) return null;

    const thumbnailV = new ThumbnailV(url);
    const imgE = thumbnailV.imgE;
    imgE.classList.add("youtube_thumb");
    imgE.src = `https://img.youtube.com/vi/${videoId}/mqdefault.jpg`;
    return thumbnailV;
}

function getYoutubeVideoId(url) {
    const regExp = /^https?:\/\/(?:www\.|m\.)?(?:youtube\.com|youtu\.be)\/(?:[\w\-]+\?v=|embed\/|v\/)?([\w\-]+)/;
    const match = url.match(regExp);
    return (match && match[1].length === 11) ? match[1] : null;
}

export function getPloudfrThumbnailV(url) {
    const regExp = /^https?:\/\/(?:www\.)?video.ploud.fr\/videos\/(?:watch|embed)\/([\w\-]+)/;
    return getPeerTubeThumbnailV(url, 'video.ploud.fr', 'ploudfr_thumb', regExp);
}

export function getLibreVideoThumbnailV(url) {
    const regExp = /^https?:\/\/(?:www\.)?libre.video\/videos\/(?:watch|embed)\/([\w\-]+)/;
    return getPeerTubeThumbnailV(url, 'libre.video', 'librevideo_thumb', regExp);
}

export function getPeertubeSocialThumbnailV(url) {
    const regExp = /^https?:\/\/(?:www\.)?peertube.social\/videos\/(?:watch|embed)\/([\w\-]+)/;
    return getPeerTubeThumbnailV(url, 'peertube.social', 'peertubesocial_thumb', regExp);
}

function getPeerTubeThumbnailV(url, domain, thumbClass, regExp) {
    const match = url.match(regExp);
    if (!match) return null;

    const videoId = match[1];
    const thumbnailV = new ThumbnailV(url);
    const imgE = thumbnailV.imgE;
    imgE.classList.add(thumbClass);
    imgE.src = `https://${domain}/static/thumbnails/${videoId}.jpg`;
    return thumbnailV;
}

export function getImgurThumbnailV(url) {
    const regExp = /^.*imgur.com\/(\w{6,7})\w?\.(\w{3})/;
    const match = regExp.exec(url);
    if (!match) return null;

    const imageId = match[1];
    const imageExt = match[2];
    const thumbId = imageId + "m";
    const thumbnailV = new ThumbnailV(url);
    const imgE = thumbnailV.imgE;
    imgE.classList.add("imgur_thumb");
    imgE.src = `https://i.imgur.com/${thumbId}.${imageExt}`;
    return thumbnailV;
}

export function getTwitterThumbnailV(url) {
    const regExp = /^(.*pbs.twimg.com\/media\/[\w_-]+)(?:\.(\w{3,4})|\?format=(\w{3,4}))/;
    const match = regExp.exec(url);
    if (!match) return null;

    const thumbnailV = new ThumbnailV(url);
    const imgE = thumbnailV.imgE;
    imgE.classList.add("twitter_thumb");
    imgE.src = `${match[1]}.${match[2] || match[3]}:thumb`;
    return thumbnailV;
}
