// file: search_form_v.js
import {config} from "../../config";
import {SearchArgs} from "../../models";
import * as location_service from "../../services/location_service";
import {showNotification} from "../notifications/notification_v";
import {textToElement} from "../../utils";

export class SearchFormV {
    constructor(searchArgs) {
        const formE = document.createElement('form');
        formE.classList.add('search_form');

        this.element = formE;
        this.searchFormV = this;

        searchArgs = searchArgs || new SearchArgs('', 'all', true, '', 0);
        this.searchArgs = searchArgs;

        const formBodyE = document.createElement('div');
        this.formBodyE = formBodyE;
        formBodyE.classList.add('form_body');
        formE.appendChild(formBodyE);

        const queryContainerE = document.createElement('div');
        queryContainerE.classList.add('inputs_container');
        formBodyE.appendChild(queryContainerE);

        const queryE = document.createElement('input');
        this.queryE = queryE;
        queryE.id = 'query';
        queryE.type = 'text';
        if (searchArgs.query) queryE.value = searchArgs.query;
        queryE.required = true;
        queryE.placeholder = 'SQL LIKE pattern (e.g. %pfpmd%, %100\\%%)';
        queryE.title = queryE.placeholder;
        queryContainerE.appendChild(queryE);

        const rangeAndBoardsContainerE = document.createElement('div');
        rangeAndBoardsContainerE.classList.add('inputs_container');
        formBodyE.appendChild(rangeAndBoardsContainerE);

        const rangeE = document.createElement('select');
        this.rangeE = rangeE;
        rangeE.id = 'range';
        rangeE.required = true;
        rangeAndBoardsContainerE.appendChild(rangeE);

        const allE = document.createElement('option');
        allE.value = 'all';
        allE.text = 'In posts and subjects';
        rangeE.add(allE);

        const postsOnlyE = document.createElement('option');
        postsOnlyE.value = 'text';
        postsOnlyE.text = 'In posts only';
        rangeE.add(postsOnlyE);

        const subjectsOnlyE = document.createElement('option');
        subjectsOnlyE.value = 'subject';
        subjectsOnlyE.text = 'In subjects only';
        rangeE.add(subjectsOnlyE);

        rangeE.value = searchArgs.range || 'all';

        const boardsE = document.createElement('input');
        this.boardsE = boardsE;
        boardsE.id = 'boards';
        if (searchArgs.boards) boardsE.value = searchArgs.boards;
        boardsE.classList.add('boards');
        boardsE.placeholder = 'In boards (e.g. b,pr,cht)';
        boardsE.title = boardsE.placeholder;
        boardsE.pattern = `${config.boardNamePattern}(?: *, *${config.boardNamePattern})*`;
        rangeAndBoardsContainerE.appendChild(boardsE);

        const orderContainerE = document.createElement('div');
        orderContainerE.classList.add('inputs_container');
        formBodyE.appendChild(orderContainerE);

        const orderE = document.createElement('select');
        this.orderE = orderE;
        orderE.id = 'order';
        orderE.required = true;
        orderContainerE.appendChild(orderE);

        const orderDesc = document.createElement('option');
        orderDesc.value = true;
        orderDesc.text = 'Order by date: desc';
        orderE.add(orderDesc);

        const orderAsc = document.createElement('option');
        orderAsc.value = false;
        orderAsc.text = 'Order by date: asc';
        orderE.add(orderAsc);

        orderE.value = searchArgs.orderDesc || true;

        const searchBtnE = document.createElement('button');
        searchBtnE.classList.add('search_btn');
        searchBtnE.classList.add('form_btn');
        searchBtnE.type = 'submit';
        searchBtnE.title = 'Ctrl+Enter';

        const searchIconE = document.createElement('i');
        searchIconE.classList.add('icon-search');
        searchBtnE.appendChild(searchIconE);
        searchBtnE.appendText('Search');
        formBodyE.appendChild(searchBtnE);

        searchBtnE.addEventListener('click', e => {
            if (!this.element.checkValidity()) return;
            e.preventDefault();

            this.searchArgs = new SearchArgs(
                this.queryE.value,
                this.rangeE.value,
                this.orderE.value,
                this.boardsE.value ? this.boardsE.value.replace(/\s/g, '') : '',
                0
            );
            location_service.setSearch(this.searchArgs);
        }, false);

        if (!config.supportSearching) {
            formE.elements.forEach(e => e.disabled = true);

            showNotification(
                config.notification.type.info,
                textToElement(
                    "This server doesn't support searching.\n" +
                    "Please try another one or use the client."
                ),
                5000
            );
        }
    }
}