// file: search_result_v.js
import {PostV} from "../post/post_v";
import {SearchResultSeparatorV} from "./search_result_separator_v";
import {PaginationV} from "../pagination/pagination_v";
import {config} from "../../config";
import {SearchResultPageLinkF} from "./search_result_page_link_f";
import {getSearchTitle, setTitle} from "../../services/title_service";
import {SearchResultPostD} from "./search_result_post_d";
import {getPageNumber} from "../../services/page_number_service";

export class SearchResultV {
    constructor(searchResultPage) {
        const searchResultE = document.createElement('div');
        this.element = searchResultE;
        searchResultE.searchV = this;
        searchResultE.classList.add(`search_result`);

        this._posts = searchResultPage.data;

        this._postsContainerE = document.createElement('div');
        this._postsContainerE.classList.add('posts_container');
        searchResultE.appendChild(this._postsContainerE);

        this._loadPosts();

        const pageLinkF = new SearchResultPageLinkF();
        const paginationV = new PaginationV(
            searchResultPage.total,
            getPageNumber(searchResultPage.skipped),
            config.pageSize.general,
            config.pagination.siblingCount,
            pageLinkF,
        );
        searchResultE.appendChild(paginationV.element);

        setTitle(getSearchTitle(searchResultPage));
    }

    onBoardDescriptionChange(boardName, boardDescription) {
        this.postVs.filter(postV => postV.post.thread.board.name === boardName).forEach(postV => {
            postV.changeBoardDescription(boardDescription);
        });
    }

    /**
     * Reloads current posts, i.e. recreates post views.
     * This is needed after, for example, some user setting has been changed.
     */
    reloadPosts() {
        const scrollTop = this.element.scrollTop;
        this._postsContainerE.empty();

        this._loadPosts();
        this.element.scrollTop = scrollTop;
    }

    _loadPosts() {
        this.postVs = this._posts.map(post => new SearchResultPostD(new PostV(post)));

        this.postVs.forEach(postV => {
            this._postsContainerE.appendChild(new SearchResultSeparatorV().element);
            this._postsContainerE.appendChild(postV.element);
        });
    }
}