// file: search_v.js
import {SearchFormV} from "./search_form_v";
import {SearchResultV} from "./search_result_v";
import {enableKatex} from "../../extra_services/katex_service";
import {enableHljs, highlightCode} from "../../extra_services/hljs_service";

export class SearchV {
    constructor(searchArgs, searchResultPage) {
        const searchE = document.createElement('div');
        this.element = searchE;
        searchE.searchV = this;
        searchE.classList.add(`search`);

        const searchTitleE = document.createElement('h3');
        searchTitleE.classList.add('search_title');
        searchTitleE.appendText('Search');
        searchE.appendChild(searchTitleE);

        const searchFormV = new SearchFormV(searchArgs);
        searchE.appendChild(searchFormV.element);

        const searchResultV = new SearchResultV(searchResultPage);
        this._searchResultV = searchResultV;
        searchE.appendChild(searchResultV.element);
    }

    onSettingsChange(oldSettings, settings) {
        return (oldSettings.isKatexEnabled !== settings.isKatexEnabled
                ? enableKatex(settings.isKatexEnabled) : Promise.resolve()
        ).then(() =>
            oldSettings.highlightCode !== settings.highlightCode
                ? enableHljs(settings.highlightCode).then() : null
        ).then(() => {
            this._searchResultV.reloadPosts();

            if (settings.highlightCode) highlightCode(this.element);
        });
    }

    onBoardDescriptionChange(boardName, boardDescription) {
        this._searchResultV.onBoardDescriptionChange(boardName, boardDescription);
    }
}