// file: settings_popup_v.js
import {PopupV} from "../popup/popup_v";
import {PopupWithCloseBtnV} from "../popup/popup_with_close_btn_v";
import {ModalPopupD} from "../popup/modal_popup_d";
import {PopupSavingStateD} from "../popup/popup_saving_state_d";
import {SettingsV} from "./settings_v";
import * as settingsService from "../../services/settings_service";
import {isEscape} from "../../services/shortcut_service";
import {setClientId} from "../../services/client_service";
import {getAccessToken} from "../../services/access_token_service";

export class SettingsPopupV {
    constructor() {
        const decoratedPopupV = new PopupV();
        decoratedPopupV.element.id = 'settings_popup';

        const closeBtnTitle = 'Esc';
        const close = () => this.close();

        const popupV = new PopupSavingStateD(
            new PopupWithCloseBtnV(
                new ModalPopupD(decoratedPopupV, close),
                close, closeBtnTitle, isEscape
            )
        );
        this._popupV = popupV;

        const popupE = popupV.element;
        this.element = popupE;
        popupE.settingsPopupV = this;

        popupE.id = 'settings_popup';
        popupE.classList.add('settings-popup');

        const titleE = document.createElement('div');
        titleE.classList.add('title');
        titleE.appendText('Settings');
        titleE.appendBefore(popupV.titleBarE.firstChild);

        this.settingsV = new SettingsV();
        popupV.popupContentE.appendChild(this.settingsV.element);

        const buttonsContainerE = document.createElement('div');
        buttonsContainerE.classList.add('buttons_container');
        popupV.popupContentE.appendChild(buttonsContainerE);

        const saveBtnE = document.createElement('button');
        saveBtnE.appendText('Save');
        saveBtnE.classList.add('save_btn');
        saveBtnE.classList.add('form_btn');
        buttonsContainerE.appendChild(saveBtnE);

        saveBtnE.addEventListener('click', () => this._save(), false);

        const cancelBtnE = document.createElement('button');
        cancelBtnE.appendText('Cancel');
        cancelBtnE.classList.add('cancel_btn');
        cancelBtnE.classList.add('form_btn');
        cancelBtnE.title = closeBtnTitle;
        buttonsContainerE.appendChild(cancelBtnE);

        cancelBtnE.addEventListener('click', () => this.close(), false);
    }

    close() {
        this._popupV.close();
    }

    _save() {
        const formE = this.settingsV.element;
        const validate = (formE.reportValidity || formE.checkValidity).bind(formE);

        if (!validate()) return;

        const settings = this.settingsV.getSettings();

        setClientId(settings.clientId);
        delete settings.clientId;

        settingsService.saveSettings(settings);
        this.close();
    }
}