// file: sidebar_board_v.js
import * as locationService from "../../services/location_service";
import {SelectableV} from "../selectable_v";
import {getBoardUrl} from "../../services/url_service";
import * as repo from "../../repository";

export class SidebarBoardV {
    constructor(board, sidebarV) {
        this.board = board;
        this.sidebarV = sidebarV;

        const itemE = document.createElement('div');
        itemE.classList.add('item');

        itemE.view = this;
        this.element = itemE;

        this.selectableV = new SelectableV(itemE);

        const favBtnE = document.createElement('div');
        this.favBtnE = favBtnE;
        itemE.appendChild(favBtnE);
        favBtnE.classList.add('fav_btn');
        favBtnE.classList.add(board.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
        favBtnE.title = board.isFavorite ? 'Remove from favorites' : 'Add to favorites';
        favBtnE.addEventListener('click', e => {
            e.preventDefault();
            this.toggleFavorite();
        }, false);

        const linkE = document.createElement('a');
        this._linkE = linkE;
        itemE.appendChild(linkE);
        linkE.classList.add('link');
        linkE.title = board.description;
        linkE.href = getBoardUrl(board.name);
        linkE.appendText(this._getLinkText());
        linkE.addEventListener('click', (e) => {
            e.preventDefault();
            locationService.setBoard(this.board.name);
        }, false);
    }

    toggleFavorite() {
        this.board.isFavorite = !this.board.isFavorite;

        repo.updateBoardIsFavorite(this.board).then(() => {
            this.toggleFavoriteUiOnly(this.board.isFavorite);
        });
    }

    toggleFavoriteUiOnly(isFavorite) {
        this.board.isFavorite = isFavorite;
        this.favBtnE.classList.remove(this.board.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
        this.favBtnE.classList.add(this.board.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
        this.favBtnE.title = this.board.isFavorite ? 'Remove from favorites' : 'Add to favorites';

        this.sidebarV.toggleFavorite(this);
    }

    setDescription(description) {
        this.board.description = description;
        this._linkE.replaceChildrenWith(this._getLinkText());
        this._linkE.title = description;
    }

    _getLinkText() {
        return `/${this.board.name}/` + (this.board.description ? ` – ${this.board.description}` : ``);
    }
}