import {decorateView, getComputedWidth} from "../../utils";

// decorates DragDependantV for sidebars
export class SidebarDragDependantD {
    constructor(dragDependantV) {
        decorateView(dragDependantV, this, 'dragDependantV');

        this.element = dragDependantV.element;
        this.element.dragDependantV = this;

        this._dragDependantV = dragDependantV;
        this._containerE = $('#container');

        const boardsSidebarE = $('#boards_sidebar');
        const threadsSidebarE = $('#threads_sidebar');
        this._isBoardsSidebar = this.element === boardsSidebarE;
        this._otherSidebarE = (this._isBoardsSidebar ? threadsSidebarE : boardsSidebarE);
    }

    setWidth(width) {
        const currentWidth = this._getWidth(this.element);
        const availableWidth = this._getAvailableWidth();
        const delta = currentWidth - width;

        // if there will be no space left
        if (availableWidth + delta < 0) {
            // This is left here just in case somebody will cry about such feature.

            // const otherSidebarWidth = getComputedWidth(this._otherSidebarE);
            // // try to use other sidebar's width if it can be cut off yet
            // if (this._isBoardsSidebar && otherSidebarWidth + delta >= 0) {
            //     this._otherSidebarE.view.setWidth(otherSidebarWidth + delta);
            // } else {
                width = currentWidth + availableWidth;
            // }
        }

        this._dragDependantV.setWidth(width);
    }

    toggleHide() {
        // in case of showing
        const totalWidth = this._getAvailableWidth() + this._getWidth(this._otherSidebarE);
        // console.log(getComputedWidth(this.element), totalWidth);
        if (this.element.isHidden() && getComputedWidth(this.element) > totalWidth) {
            this._dragDependantV.setWidth(totalWidth);
        }

        this._dragDependantV.toggleHide();

        if (!this._otherSidebarE.isHidden() && this._getAvailableWidth() === 0) {
            this._otherSidebarE.dragDependantV
                .setWidth(totalWidth - getComputedWidth(this.element));
        }
    }

    _getAvailableWidth() {
        // return getComputedWidth(this._containerE);
        return this._containerE.offsetWidth;
    }

    _getWidth(element) {
        // NOTE: if the element is hidden, use getComputedWidth() if you don't want 0.
        return element.offsetWidth;
    }
}