// file: sidebar_thread_v.js
import * as locationService from "../../services/location_service";
import {SelectableV} from "../selectable_v";
import {getThreadUrl} from "../../services/url_service";
import * as repo from "../../repository";
import {toShortcutUuid} from "../../services/uuid_service";

export class SidebarThreadV {
    constructor(thread, sidebarV) {
        this.thread = thread;
        this.sidebarV = sidebarV;

        const itemE = document.createElement('div');
        itemE.classList.add('item');

        itemE.view = this;
        this.element = itemE;

        this.selectableV = new SelectableV(itemE);

        this.favBtnE = document.createElement('div');
        itemE.appendChild(this.favBtnE);
        this.favBtnE.classList.add('fav_btn');
        this.favBtnE.classList.add(thread.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
        this.favBtnE.title = thread.isFavorite ? 'Remove from favorites' : 'Add to favorites';
        this.favBtnE.addEventListener('click', e => {
            e.preventDefault();
            this.toggleFavorite();
        }, false);

        const linkE = document.createElement('a');
        itemE.appendChild(linkE);
        linkE.classList.add('link');
        linkE.title = thread.subject;
        linkE.href = getThreadUrl(thread.board.name, thread.threadId);

        const boardPrefix = locationService.showAllThreads() ? `/${thread.board.name}/` : ``;

        linkE.appendText(`${boardPrefix}${toShortcutUuid(thread.threadId)} – ${thread.subject}`);

        linkE.addEventListener('click', e => {
            e.preventDefault();
            locationService.setThread(this.thread.board.name, this.thread.threadId);
        }, false);
    }

    toggleFavorite() {
        this.thread.isFavorite = !this.thread.isFavorite;

        repo.updateThreadIsFavorite(this.thread).then(() => {
            this.toggleFavoriteUiOnly(this.thread.isFavorite);
        });
    }

    toggleFavoriteUiOnly(isFavorite) {
        this.thread.isFavorite = isFavorite;
        this.favBtnE.classList.remove(this.thread.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
        this.favBtnE.classList.add(this.thread.isFavorite ? 'icon-fav_remove' : 'icon-fav_add');
        this.favBtnE.title = this.thread.isFavorite ? 'Remove from favorites' : 'Add to favorites';

        this.sidebarV.toggleFavorite(this);
    }
}