// file: textbox_v.js
export class TextboxV {
    constructor(afterResize = () => {}) {
        const tbxE = document.createElement('textarea');
        tbxE.setAttribute("type", "text");

        this.isAutoResizable = false;
        this.element = tbxE;
        this.afterResize = afterResize;

        this._shrinkable = true;
        this._textHeights = new Set();
    }

    insertText(text, focus = true) {
        const tbxE = this.element;
        const oldText = tbxE.value;
        const selectionEnd = tbxE.selectionEnd;

        const firstPart = oldText.slice(0, tbxE.selectionEnd);
        const secondPart = oldText.slice(tbxE.selectionEnd);
        tbxE.value = (firstPart + text + secondPart);

        tbxE.selectionEnd = selectionEnd + text.length;
        if (focus) tbxE.focus();
        if (this.isAutoResizable) this.resize();
    }

    getTextBeforeSelectionEnd() {
        const tbxE = this.element;
        return tbxE.value.slice(0, tbxE.selectionEnd);
    }

    resize() {
        if (this._isResizing) return;

        this._isResizing = true;
        const tbxE = this.element;

        const shrink = (
            this._shrinkable &&
            tbxE.scrollHeight === tbxE.clientHeight &&
            this._previousValue.length > tbxE.value.length
        );
        this._previousValue = tbxE.value;

        requestAnimationFrame(() => {
            let scrollHeight = tbxE.scrollHeight;

            if (!shrink) {
                this._textHeights.add(scrollHeight);
            }
            if (shrink) {
                // to prevent bug @80554
                // first fix parent element
                tbxE.parentNode.style.minHeight = `${tbxE.parentNode.scrollHeight}px`;

                tbxE.style.height = '';

                // scrollHeight is rounded to integer
                // so sometimes it might be wrong after shrinking by 1px
                if (this._textHeights.has(tbxE.scrollHeight)) {
                    scrollHeight = tbxE.scrollHeight;
                } else {
                    scrollHeight = tbxE.scrollHeight + 1;
                }
            }

            tbxE.style.height = (scrollHeight + tbxE.offsetHeight - tbxE.clientHeight) + "px";
            this._isResizing = false;
            this.afterResize();

            if (shrink) {
                // release parent
                tbxE.parentNode.style.minHeight = null;
            }
        });
    }

    autoResize(shrinkable = true) {
        this.isAutoResizable = true;
        this._previousValue = this.element.value;
        this._shrinkable = shrinkable;

        this.element.addEventListener("input", () => this.resize(), false);
    }

    forceResize() {
        this._previousValue = `${this.element.value} `;
        this.resize();
    }

    clear() {
        this.element.value = "";
        if (this.isAutoResizable) this.resize();
    }
}
