// file: shell_thread_v.js
import {ThreadBannerQuotedSubjD} from "../banners/thread_banner_quoted_subj_d";
import {ThreadBannerV} from "../banners/thread_banner_v";
import {ThreadBannerWithBoardLinkD} from "./thread_banner_with_board_link_d";
import {ReplyFormV} from "../reply_form/reply_form_v";
import {ThreadSummaryV} from "./thread_summary_v";
import {ThreadBannerWithThreadLinkD} from "./thread_banner_with_thread_link_d";
import {createThreadPostV} from "../thread_page/thread_page_v";
import {ReplyFormPositionedD} from "../reply_form/reply_form_positioned_d";
import {config} from "../../config";
import * as trackNewPostsService from "../../services/track_new_posts_service";

export class ShellThreadV {
    constructor(shellThread, isGlobalWarp) {
        const shellThreadE = document.createElement('div');
        shellThreadE.classList.add('shell_thread');

        this.element = shellThreadE;
        shellThreadE.shellThreadV = this;

        this.isGlobalWarp = isGlobalWarp;
        this.shellThread = shellThread;

        this._thread = shellThread.thread;
        this._thread.board = this._thread.board || {};

        this._thread.lastSeenAt = config.isClientWebUI
            ? this._thread.lastSeenAt
            : trackNewPostsService.getLastSeenAt(this._thread.threadId);

        this.threadBannerV = new ThreadBannerV(this._thread.threadId, this._thread.subject);
        this.threadBannerV = new ThreadBannerQuotedSubjD(this.threadBannerV);
        this.threadBannerV = new ThreadBannerWithThreadLinkD(
            this.threadBannerV, this._thread.board.name, this._thread.threadId
        );

        if (isGlobalWarp) {
            this.threadBannerV = new ThreadBannerWithBoardLinkD(
                this.threadBannerV, this._thread.board.name, this._thread.board.description
            );
        }
        shellThreadE.appendChild(this.threadBannerV.element);

        const replyFormV = new ReplyFormPositionedD(
            new ReplyFormV(this._thread.threadId, this._thread.board.name)
        );
        this.replyFormV = replyFormV;

        replyFormV.element.hide();

        this._loadPosts();

        shellThreadE.appendChild(replyFormV.element);
    }

    onNewPost(post) {
    }

    onUpdatedHiddenByOp(hiddenByOpEvent) {
        this.shellThread.posts.filter(post => post.postId === hiddenByOpEvent.postId)
            .forEach(post => post.hiddenByOp = hiddenByOpEvent.hiddenByOp);

        this.reloadPosts();
    }

    onBoardDescriptionChange(boardName, boardDescription) {
        if (boardName !== this._thread.board.name) return;

        this._changeBoardDescription(boardDescription);
    }

    _changeBoardDescription(boardDescription) {
        this._thread.board.description = boardDescription;

        if (this.isGlobalWarp) {
            this.threadBannerV.changeBoardDescription(boardDescription);
        }
    }

    reloadPosts() {
        this.element.children.filter(e => e.postV || e.summaryV).forEach(e => e.remove());
        this._loadPosts();
    }

    _loadPosts() {
        const op = this.shellThread.op;

        this.originalPostV = this._createThreadPostV(op, true);
        this.element.appendChild(this.originalPostV.element);

        const threadSummaryV = new ThreadSummaryV(this._thread, this.shellThread.posts.length + 1);
        this.element.appendChild(threadSummaryV.element);

        this.postVs = this.shellThread.posts.map(post => this._createThreadPostV(post, false));
        this.postVs.forEach(postV => this.element.appendChild(postV.element));
    }

    _createThreadPostV(post, isOriginalPost) {
        return createThreadPostV(post, isOriginalPost, this.replyFormV, this._thread.lastSeenAt);
    }
}