// file: warp_page_v.js
import {ShellThreadV} from "./shell_thread_v";
import {ThreadsSeparatorV} from "./threads_separator_v";
import {PaginationV} from "../pagination/pagination_v";
import {WarpPageLinkF} from "./warp_page_link_f";
import {getWarpTitle, setTitle} from "../../services/title_service";
import {NewThreadFormV} from "./new_thread_form_v";
import {BoardBannerV} from "../banners/board_banner_v";
import {config} from "../../config";
import {enableKatex} from "../../extra_services/katex_service";
import {enableHljs, highlightCode} from "../../extra_services/hljs_service";
import {getPageNumber} from "../../services/page_number_service";

export class WarpPageV {
    constructor(warpPage) {
        this.warpPage = warpPage;

        const warpPageE = document.createElement('div');
        warpPageE.classList.add('warp');

        this.element = warpPageE;
        warpPageE.threadPageV = this;

        const isGlobalWarp = !warpPage.data.board;
        const isChatBoard = !isGlobalWarp && warpPage.data.board.name === config.chat.boardName;
        this._isChatBoard = isChatBoard;

        if (!isGlobalWarp) {
            const boardBannerV = new BoardBannerV(
                warpPage.data.board.name,
                warpPage.data.board.description,
            );
            this._boardBannerV = boardBannerV;
            warpPageE.appendChild(boardBannerV.element);

            if (!isChatBoard) {
                const threadsSeparatorV = new ThreadsSeparatorV();
                this._threadsSeparatorV = threadsSeparatorV;
                warpPageE.appendChild(threadsSeparatorV.element);
            }
        }

        if (!isChatBoard) {
            const startNewThreadE = document.createElement('h3');
            this._startNewThreadE = startNewThreadE;
            startNewThreadE.classList.add('start_new_thread');
            startNewThreadE.appendText('Start a New Thread');
            warpPageE.appendChild(startNewThreadE);

            // const newThreadBtnContainerE = document.createElement('div');
            // newThreadBtnContainerE.classList.add('start_new_thread_btn_container');
            // warpPageE.appendChild(newThreadBtnContainerE);
            //
            // const newThreadBtnE = document.createElement('button');
            // newThreadBtnE.appendText('Start a New Thread');
            // warpPageE.appendChild(newThreadBtnE);

            const newThreadFormV = new NewThreadFormV(
                isGlobalWarp ? null : warpPage.data.board.name
            );
            this.newThreadFormV = newThreadFormV;
            const newThreadFormE = newThreadFormV.element;
            // newThreadFormE.hide();
            warpPageE.appendChild(newThreadFormV.element);

            // newThreadBtnE.addEventListener('click', e => {
            //     newThreadFormE.toggleHide();
            //     newThreadBtnE.toggleHide();
            //
            //     // newThreadBtnE.appendText(newThreadFormE.isHidden()
            //     //     ? 'Start a New Thread' : 'Cancel'
            //     // );
            // }, false);
        }

        this.shellThreadsVs = warpPage.data.shellThreads.map(shellThread => {
            const threadsSeparatorV = new ThreadsSeparatorV();
            warpPageE.appendChild(threadsSeparatorV.element);

            const shellThreadV = new ShellThreadV(shellThread, isGlobalWarp);
            warpPageE.appendChild(shellThreadV.element);
            return shellThreadV;
        });

        const pageLinkF = new WarpPageLinkF(warpPage);
        const paginationV = new PaginationV(
            warpPage.total,
            getPageNumber(warpPage.skipped, config.pageSize.warp),
            config.pageSize.warp,
            config.pagination.siblingCount,
            pageLinkF,
        );
        warpPageE.appendChild(paginationV.element);

        setTitle(getWarpTitle(warpPage));
    }

    onSettingsChange(oldSettings, settings) {
        return (oldSettings.isKatexEnabled !== settings.isKatexEnabled
                ? enableKatex(settings.isKatexEnabled) : Promise.resolve()
        ).then(() =>
            oldSettings.highlightCode !== settings.highlightCode
                ? enableHljs(settings.highlightCode).then() : null
        ).then(() => {
            this.reloadPosts();

            if (settings.highlightCode) highlightCode(this.element);
        });
    }

    reloadPosts() {
        this.shellThreadsVs.forEach(shellThread => shellThread.reloadPosts());
    }

    onNewPost(post) {
        this.shellThreadsVs.forEach(shellThread => shellThread.onNewPost(post));
    }

    onUpdatedHiddenByOp(hiddenByOpEvent) {
        this.shellThreadsVs.forEach(shellThread => shellThread.onUpdatedHiddenByOp(hiddenByOpEvent));
    }

    onBoardDescriptionChange(boardName, boardDescription) {
        const board = this.warpPage.data.board;

        if (board && board.name !== boardName) return;

        this._changeBoardDescription(boardDescription);

        this.shellThreadsVs.forEach(
            shellThreadV => shellThreadV.onBoardDescriptionChange(boardName, boardDescription)
        );
    }

    _changeBoardDescription(boardDescription) {
        const board = this.warpPage.data.board;
        if (!board) return;

        board.description = boardDescription;

        if (!this._boardBannerV) return;

        this._boardBannerV.changeBoardDescription(boardDescription);
    }
}