// file: warp_page_with_state_d.js
import {decorateView} from "../../utils";

export class WarpPageWithStateD {
    constructor(warpPageV) {
        decorateView(warpPageV, this, 'warpPageV');

        this._warpPageV = warpPageV;
        this.element = warpPageV.element;
    }

    get newThreadFormV() {
        return this._warpPageV.newThreadFormV;
    }

    get shellThreadsVs() {
        return this._warpPageV.shellThreadsVs;
    }

    getState() {
        return new WarpPageState(
            this._getNewThreadFormState(),
            this.shellThreadsVs.map(shellThreadV => this._getShellThreadState(shellThreadV))
        );
    }

    restoreState(warpPageState) {
        if (warpPageState.newThreadFormState) {
            this._restoreNewThreadFormState(warpPageState.newThreadFormState);
        }

        warpPageState.shellThreadsStates.forEach(
            shellThreadState => {
                this._restoreShellThreadState(shellThreadState.threadId, shellThreadState);
            }
        );
    }

    _getNewThreadFormState() {
        if (!this.newThreadFormV) return null;

        return new NewThreadFormState(
            this.newThreadFormV.boardE ? this.newThreadFormV.boardE.value : null,
            this.newThreadFormV.subjectE.value,
            this.newThreadFormV.tbxV.element.value
        );
    }

    _getShellThreadState(shellThreadV) {
        const postAbove = shellThreadV.replyFormV.postAbove;
        const replyFormAfterPostId = postAbove ? postAbove.postId : null;

        return new ShellThreadState(
            shellThreadV.shellThread.thread.threadId,
            replyFormAfterPostId,
            shellThreadV.replyFormV.tbxV.element.value
        );
    }

    _restoreNewThreadFormState(newThreadFormState) {
        if (newThreadFormState.boardName) {
            this.newThreadFormV.boardE.value = newThreadFormState.boardName;
        }

        if (newThreadFormState.subject) {
            this.newThreadFormV.subjectE.value = newThreadFormState.subject;
        }
        if (newThreadFormState.text) {
            this.newThreadFormV.tbxV.element.value = newThreadFormState.text;
        }
    }

    _restoreShellThreadState(threadId, shellThreadState) {
        const shellThreadV = this.shellThreadsVs.find(
            shellThreadV => shellThreadV.shellThread.thread.threadId === threadId
        );

        if (!shellThreadV) return; // probably that thread is not on this page anymore

        const postsVs = [shellThreadV.originalPostV].concat(shellThreadV.postVs);

        if (shellThreadState.replyFormAfterPostId) {
            (postsVs.find(postV => postV.post.postId === shellThreadState.replyFormAfterPostId)
                || postsVs.last()
            ).reply(false);
        }

        shellThreadV.replyFormV.tbxV.element.value = shellThreadState.text;
    }
}

class WarpPageState {
    constructor(newThreadFormState, shellThreadsStates) {
        this.newThreadFormState = newThreadFormState;
        this.shellThreadsStates = shellThreadsStates;
    }
}

class NewThreadFormState {
    constructor(boardName, subject, text) {
        this.boardName = boardName;
        this.subject = subject;
        this.text = text;
    }
}

class ShellThreadState {
    constructor(threadId, replyFormAfterPostId, text) {
        this.threadId = threadId;
        this.replyFormAfterPostId = replyFormAfterPostId;
        this.text = text;
    }
}